import json
import pytest
import re


@pytest.mark.prometheus
def test_prometheus_alerts(host):
    """Test prometheus for firing alerts."""

    def summarize_alerts(alerts):
        """Print a alert summary."""
        print('Total alerts: %s' % len(alerts))
        print(json.dumps(alerts, indent=2))

    print("Starting prometheus test...")

    prometheus_ip = host.check_output(
        'dig +short '
        'monitoring-prometheus-oper-prometheus.oas.svc.cluster.local '
        '@10.43.0.10')
    url = 'http://{0}:9090/api/v1/alerts'.format(prometheus_ip)
    alert_json = json.loads(host.check_output('curl ' + url))
    status = alert_json["status"]
    alerts = alert_json["data"]["alerts"]

    real_alerts = []
    ignored_alerts = []

    for alert in alerts:
        # Filter out the ever firing "Dead mans switch" test alert
        if ignore_alert(alert):
            ignored_alerts.append(alert)
        else:
            real_alerts.append(alert)

    print('\n\n\n========= Ignored ==========')
    summarize_alerts(ignored_alerts)

    print('\n\n\n========= Firing ==========')
    summarize_alerts(real_alerts)

    count = len(real_alerts)

    assert status == "success", "Failure queriying the prometheus api at" + url
    assert count == 0, "Firing alerts: {0}".format(count)


def ignore_alert(alert):
    """
    Returns true if an alert should be ignored. This is when the "severity"
    equals "none", or in some application specific cases.
    """

    if alert["labels"]["severity"] == "none":
        return True

    # Ignore `KubeAPILatencyHigh` fom high load during installation
    # phase
    if alert["labels"]["alertname"] == "KubeAPILatencyHigh":
        return True

    if 'pod' in alert["labels"]:
        # Filter out failing Nextcloud installation jobs since a lot of
        # them fail until they succeed during installation
        if "nextcloud" in alert["labels"]["pod"]:
            if alert["labels"]["alertname"] in [
                    "KubeJobFailed",
                    "KubeJobCompletion",
                    "KubePodNotReady"]:
                return True

        # Filter out when some apps take too long to start
        if re.search("(rocketchat|wordpress)", alert["labels"]["pod"]):
            if alert["labels"]["alertname"] in [
                    "KubePodNotReady",
                    "KubeDeploymentReplicasMismatch",
                    "KubeStatefulSetReplicasMismatch"]:
                return True

        # Filter out failed signgle-sign-on pods until we fix
        # https://open.greenhost.net/openappstack/single-sign-on/issues/26
        if "single-sign-on-create-" in alert["labels"]["pod"]:
            if alert["labels"]["alertname"] in ["KubePodNotReady"]:
                return True

    if 'job_name' in alert["labels"]:
        # Filter out failed signgle-sign-on jobs until we fix
        # https://open.greenhost.net/openappstack/single-sign-on/issues/26
        if "single-sign-on-create-" in alert["labels"]["job_name"]:
            if alert["labels"]["alertname"] in [
                    "KubeJobFailed",
                    "KubeJobCompletion"]:
                return True

    return False


if __name__ == "__main__":
    test_prometheus_alerts('')
